@@ -2,7 +2,11 @@
 # Distributed under the terms of the Modified BSD License.
 
 from .app import LabServerApp
+from .licenses_app import LicensesApp
 from .handlers import add_handlers, LabHandler, LabConfig
+from .spec import get_openapi_spec, get_openapi_spec_dict
+from .translation_utils import translator
+from .workspaces_app import WorkspaceExportApp, WorkspaceImportApp, WorkspaceListApp
 from .workspaces_handler import slugify, WORKSPACE_EXTENSION
 from ._version import __version__
 
@@ -12,9 +16,14 @@ __all__ = [
     'LabConfig',
     'LabHandler',
     'LabServerApp',
-    'slugify',
+    'LicensesApp',
     'SETTINGS_EXTENSION',
-    'WORKSPACE_EXTENSION'
+    'slugify',
+    'translator',
+    'WORKSPACE_EXTENSION',
+    'WorkspaceExportApp',
+    'WorkspaceImportApp',
+    'WorkspaceListApp'
 ]
 
 def _jupyter_server_extension_points():
@@ -23,4 +32,4 @@ def _jupyter_server_extension_points():
             'module': 'jupyterlab_server',
             'app': LabServerApp
         }
-    ]
\ No newline at end of file
+    ]
