@@ -5,6 +5,7 @@
 # Distributed under the terms of the Modified BSD License.
 import os
 from urllib.parse import urlparse
+from functools import lru_cache
 
 from tornado import template, web
 
@@ -17,7 +18,7 @@ from .server import url_path_join as ujoin
 from .settings_handler import SettingsHandler
 from .themes_handler import ThemesHandler
 from .translations_handler import TranslationsHandler
-from .workspaces_handler import WorkspacesHandler
+from .workspaces_handler import WorkspacesHandler, WorkspacesManager
 from .licenses_handler import LicensesHandler, LicensesManager
 
 # -----------------------------------------------------------------------------
@@ -57,18 +58,13 @@ def is_url(url):
 class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandler):
     """Render the JupyterLab View."""
 
-    @web.authenticated
-    @web.removeslash
-    def get(self, mode = None, workspace = None, tree = None):
-        """Get the JupyterLab html page."""
-        workspace = 'default' if workspace is None else workspace.replace('/workspaces/','')
-        tree_path = '' if tree is None else tree.replace('/tree/','')
-
+    @lru_cache()
+    def get_page_config(self):
+        """Construct the page config object"""
         self.application.store_id = getattr(self.application, 'store_id', 0)
         config = LabConfig()
         app = self.extensionapp
         settings_dir = app.app_settings_dir
-
         # Handle page config data.
         page_config = self.settings.setdefault('page_config_data', {})
         terminals = self.settings.get('terminals_available', False)
@@ -76,7 +72,7 @@ class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandl
         server_root = server_root.replace(os.sep, '/')
         base_url = self.settings.get('base_url')
 
-        # Remove the trailing slash for compatibiity with html-webpack-plugin.
+        # Remove the trailing slash for compatibility with html-webpack-plugin.
         full_static_url = self.static_url_prefix.rstrip('/')
         page_config.setdefault('fullStaticUrl', full_static_url)
 
@@ -84,6 +80,17 @@ class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandl
         page_config.setdefault('ignorePlugins', [])
         page_config.setdefault('serverRoot', server_root)
         page_config['store_id'] = self.application.store_id
+
+        server_root = os.path.normpath(os.path.expanduser(server_root))
+        try:
+            # Remove the server_root from pref dir
+            if self.serverapp.preferred_dir != server_root:
+                page_config['preferredPath'] = '/' + os.path.relpath(self.serverapp.preferred_dir, server_root)
+            else:
+                page_config['preferredPath'] = '/'
+        except Exception:
+            page_config['preferredPath'] = '/'
+
         self.application.store_id += 1
 
         mathjax_config = self.settings.get('mathjax_config',
@@ -96,14 +103,6 @@ class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandl
         page_config.setdefault('mathjaxConfig', mathjax_config)
         page_config.setdefault('fullMathjaxUrl', mathjax_url)
 
-        # Add parameters parsed from the URL
-        if mode == 'doc':
-            page_config['mode'] = 'single-document'
-        else:
-            page_config['mode'] = 'multiple-document'
-        page_config['workspace'] = workspace
-        page_config['treePath'] = tree_path
-
         # Put all our config in page_config
         for name in config.trait_names():
             page_config[_camelCase(name)] = getattr(app, name)
@@ -123,17 +122,43 @@ class LabHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandl
         labextensions_path = app.extra_labextensions_path + app.labextensions_path
         recursive_update(page_config, get_page_config(labextensions_path, settings_dir, logger=self.log))
 
+        # modify page config with custom hook
+        page_config_hook = self.settings.get("page_config_hook", None)
+        if page_config_hook:
+            page_config = page_config_hook(self, page_config)
+
+        return page_config
+
+    @web.authenticated
+    @web.removeslash
+    def get(self, mode=None, workspace=None, tree=None):
+        """Get the JupyterLab html page."""
+        workspace = (
+            "default" if workspace is None else workspace.replace("/workspaces/", "")
+        )
+        tree_path = "" if tree is None else tree.replace("/tree/", "")
+
+        page_config = self.get_page_config()
+
+        # Add parameters parsed from the URL
+        if mode == "doc":
+            page_config["mode"] = "single-document"
+        else:
+            page_config["mode"] = "multiple-document"
+        page_config["workspace"] = workspace
+        page_config["treePath"] = tree_path
+
         # Write the template with the config.
         tpl = self.render_template('index.html', page_config=page_config)
         self.write(tpl)
 
 
 class NotFoundHandler(LabHandler):
-    def render_template(self, name, **ns):
-        if 'page_config' in ns:
-            ns['page_config'] = ns['page_config'].copy()
-            ns['page_config']['notFoundUrl'] = self.request.path
-        return super().render_template(name, **ns)
+    @lru_cache()
+    def get_page_config(self):
+        page_config = super().get_page_config()
+        page_config["notFoundUrl"] = self.request.path
+        return page_config
 
 
 def add_handlers(handlers, extension_app):
@@ -192,11 +217,24 @@ def add_handlers(handlers, extension_app):
         setting_path = ujoin(extension_app.settings_url, '(?P<schema_name>.+)')
         handlers.append((setting_path, SettingsHandler, settings_config))
 
+        # Handle translations.
+        ## Translations requires settings as the locale source of truth is stored in it
+        if extension_app.translations_api_url:
+            # Handle requests for the list of language packs available.
+            # Make slash optional.
+            translations_path = ujoin(extension_app.translations_api_url, '?')
+            handlers.append((translations_path, TranslationsHandler, settings_config))
+
+            # Handle requests for an individual language pack.
+            translations_lang_path = ujoin(
+                extension_app.translations_api_url, '(?P<locale>.*)')
+            handlers.append((translations_lang_path, TranslationsHandler, settings_config))
+
     # Handle saved workspaces.
     if extension_app.workspaces_dir:
 
         workspaces_config = {
-            'path': extension_app.workspaces_dir
+            'manager': WorkspacesManager(extension_app.workspaces_dir)
         }
 
         # Handle requests for the list of workspaces. Make slash optional.
@@ -270,18 +308,6 @@ def add_handlers(handlers, extension_app):
             }
         ))
 
-    # Handle translations.
-    if extension_app.translations_api_url:
-        # Handle requests for the list of language packs available.
-        # Make slash optional.
-        translations_path = ujoin(extension_app.translations_api_url, '?')
-        handlers.append((translations_path, TranslationsHandler, {'lab_config': extension_app}))
-
-        # Handle requests for an individual language pack.
-        translations_lang_path = ujoin(
-            extension_app.translations_api_url, '(?P<locale>.*)')
-        handlers.append((translations_lang_path, TranslationsHandler, {'lab_config': extension_app}))
-
     # Let the lab handler act as the fallthrough option instead of a 404.
     fallthrough_url = ujoin(extension_app.app_url, r'.*')
     handlers.append((fallthrough_url, NotFoundHandler))
